# -*- python -*-
#
# Copyright 2018, 2019, 2020, 2021 Liang Chen
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from setuptools import find_packages, setup


class PkgSetup(object):
    NAME = 'dmprj'
    DESCRIB = 'Dedicated Modeling Project'
    LICENSE = 'Apache License 2.0'
    AUTHOR = 'Liang Chen'

    CLASSIFIERS = [
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ]

    KEYWORDS = ('Modeling', 'Abstract')

    @classmethod
    def read_src_file(cls, *parts):
        import codecs
        import os
        dir_path = os.path.abspath(os.path.dirname(__file__))
        with codecs.open(os.path.join(dir_path, *parts), 'r') as _f:
            return _f.read()

    @classmethod
    def find_version(cls, *file_paths):
        import re

        MSG_VERSION_STRING_NOTFOUND = 'Unable to find version string.'

        version_file = cls.read_src_file(*file_paths)
        version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", version_file, re.M)
        if version_match:
            return version_match.group(1)
        raise RuntimeError(MSG_VERSION_STRING_NOTFOUND)

    @classmethod
    def getConfig(cls):
        cfg = {
            'name': cls.NAME,
            'author': cls.AUTHOR,
            'license': cls.LICENSE,
            'classifiers': cls.CLASSIFIERS,
            'keywords': cls.KEYWORDS,
            'version': cls.find_version(cls.NAME, '__init__.py'),
            'packages': find_packages(),
            'platforms': 'Any',
            'python_requires': '>=2.7',
            #'python_requires': '>=3.7',
            'install_requires': [],
            'tests_require': ['flake8', 'pytest'],
            'description': cls.DESCRIB,
            'long_description': cls.read_src_file('README.md'),
            'long_description_content_type': 'text/markdown',
        }
        return cfg


setup(**(PkgSetup.getConfig()))
